VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "CFiber"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'Definiert, wieviele Fibers maximal verwendet werden knnen
Const MaxFibers As Long = 20

'Definiert die Zeitscheibenlnge eines Fibers
Const MaxDuration As Long = 20 'ms

'API Imports
Private Declare Sub GetSystemTimeAsFileTime Lib "kernel32.dll" (ByRef lpSystemTimeAsFileTime As Currency)

Private Declare Function W32CreateFiber Lib "kernel32.dll" Alias "CreateFiber" (ByVal dwStackSize As Long, ByVal lpStartAddress As Long, ByRef lpParameter As Any) As Long
Private Declare Sub W32DeleteFiber Lib "kernel32.dll" Alias "DeleteFiber" (ByVal lpFiber As Long)
Private Declare Sub W32SwitchToFiber Lib "kernel32.dll" Alias "SwitchToFiber" (ByVal lpFiber As Long)

Private Declare Function W32ConvertThreadToFiber Lib "kernel32.dll" Alias "ConvertThreadToFiber" (ByVal lpFiber As Long) As Long
Private Declare Function W32CConvertFiberToThread Lib "kernel32.dll" Alias "ConvertFiberToThread" (ByVal lpFiber As Long) As Long

Private Declare Sub W32Sleep Lib "kernel32.dll" Alias "Sleep" (ByVal dwMilliseconds As Long)

'Runtime Daten
Private InitVar As Long

Private Type DataType
    SecCount As Long
    FiberCount As Long
    CurrentFiber As Long
    NextFiberId As Long
    MaxFiberCount As Long
    TimeSliceDuration As Currency
    NextSchedule As Currency
    Fibers(0 To (MaxFibers - 1)) As FiberData
End Type

Private Data As DataType

Private Sub Class_Initialize()

'Fiber-Threading beim System anmelden und aktuellen Fiber als Fiber 0 speichern
Data.Fibers(0).Fiber = W32ConvertThreadToFiber(InitVar)
If (Data.Fibers(0).Fiber <> 0) Then

    Data.NextFiberId = 1
    
    Data.TimeSliceDuration = CCur(MaxDuration)
    Data.MaxFiberCount = MaxFibers
    
    Call GetSystemTimeAsFileTime(Data.Fibers(0).LastTimeSlice)
    
Else
    Call Err.Raise(2, , "can't convert thread to fiber")
    End
End If

End Sub

'Fiber-Threading beenden. Muss von Fiber 0(!) veranlasst werden. D.h. Klasse darf nur aus dem Kontext
'entladen werden, in dem sie auch geladen wurde!
Private Sub Class_Terminate()

Dim i As Long

'Alle Fibers aufrumen
For i = 1 To (MaxFibers - 1)
    If (Data.Fibers(i).Fiber <> 0) Then
        Call W32DeleteFiber(Data.Fibers(i).Fiber)
    End If
Next

End Sub

'Erzeugt einen neuen Fiber. Entry ist die Startfunktion, Param ein beliebiger Parameter. Es wird die ID des neuen
'Fibers zurckgegeben bzw. 0 im Fehlerfall
Public Function CreateFiber(ByVal Entry As Long, ByVal Param As Long) As Long

Dim i As Long
    
'Freien Slot suchen...
For i = 1 To (MaxFibers - 1)
    If (Data.Fibers(i).Fiber = 0) Then
    
        '... und Fiber sowie Daten zur Verwaltung anlegen
        Data.Fibers(i).Id = Data.NextFiberId
        Data.Fibers(i).UserProc = Entry
        Data.Fibers(i).UserParam = Param
        Data.Fibers(i).DeletionPending = False

        Data.Fibers(i).Fiber = W32CreateFiber(0, AddressOf FiberMainProc, Data.Fibers(i))

        CreateFiber = Data.Fibers(i).Id
        Data.NextFiberId = (Data.NextFiberId + 1)
        Data.FiberCount = (Data.FiberCount + 1)
        
        Exit For
    End If
Next
    
End Function

'Lscht einen Fiber. Kann nicht auf den Fiber 0 angewendet werden.
Public Sub DeleteFiber(ByVal Fiber As Long)

For i = 1 To (MaxFibers - 1)
    If (Data.Fibers(i).Fiber <> 0) Then
        If (Data.Fibers(i).Id = Fiber) Then
        
            If (Data.CurrentFiber <> Fiber) Then
                Call W32DeleteFiber(Data.Fibers(i).Fiber)
                Data.Fibers(i).Fiber = 0
                Data.FiberCount = (Data.FiberCount - 1)
            Else
                Data.Fibers(i).DeletionPending = True
                Data.NextSchedule = 0
                Call Schedule
            End If
            
            Exit For
        End If
    End If
Next

End Sub

'Wechselt den Kontext zum angegebenen Fiber
Public Sub SwitchToFiber(ByVal Fiber As Long)

Dim i As Long

For i = 0 To (MaxFibers - 1)
    If (Data.Fibers(i).Fiber <> 0) Then
        If (Data.Fibers(i).Id = Fiber) Then
                        
            Call GetSystemTimeAsFileTime(Data.Fibers(i).LastTimeSlice)
    
            'Nchsten Ablaufzeitpunkt berechnen
            Data.NextSchedule = (Data.Fibers(i).LastTimeSlice + Data.TimeSliceDuration)
    
            If (Data.Fibers(i).Id <> Data.CurrentFiber) Then
                Data.CurrentFiber = Data.Fibers(i).Id
                Call W32SwitchToFiber(Data.Fibers(i).Fiber)
            End If
                
            Exit For
        End If
    End If
Next

End Sub

'Setzt neue Dauer (Zeitscheibe) in ms bis zum Kontextwechsel
Public Sub SetTimeSliceDuration(ByVal Duration As Long)

Data.TimeSliceDuration = CCur(Duration)

End Sub

'Gibt die ID des aktiven Fiber zurck
Public Function GetCurrentFiber() As Long

GetCurrentFiber = Data.CurrentFiber

End Function

'Veranlasst bei Bedarf einen Kontextwechsel, Gibt true zurck, wenn eine Zeitscheibe abgelaufen
'ist und ein Scheduling veranlasst wird
Public Function Schedule() As Long

Dim i As Long, j As Long
Dim T As Currency

Call GetSystemTimeAsFileTime(T)

'Testen, ob aktuelle Zeitscheibe abgelaufen ist
If (Data.NextSchedule <= T) Then
    
    'Fiber suchen, dem am lngsten keine Rechenzeit zugewiesen wurde
    For i = 0 To (MaxFibers - 1)
        If (Data.Fibers(i).SleepTimeout <= T) Then
            j = i
            Exit For
        End If
    Next
    
    If (i < MaxFibers) Then
        
        For i = 0 To (MaxFibers - 1)
            If (Data.Fibers(i).Fiber <> 0) Then
                If (Data.Fibers(i).Id <> Data.CurrentFiber) Then
                    If (Data.Fibers(i).DeletionPending = True) Then
                        Call DeleteFiber(Data.Fibers(i).Id)
                    Else
                        If (Data.Fibers(i).SleepTimeout < T) Then
                            If (Data.Fibers(i).LastTimeSlice <= Data.Fibers(j).LastTimeSlice) Then
                                j = i
                            End If
                        End If
                    End If
                End If
            End If
        Next
        
        'Zum gefundenen Fiber wechseln
        Call SwitchToFiber(Data.Fibers(j).Id)
            
        Schedule = True
        
    End If
    
End If

End Function

'Gibt die Anzahl der momentan existierenden Fiber zurck
Public Function GetFiberCount() As Long

GetFiberCount = Data.FiberCount

End Function

'Alle Fibers lschen.
Public Sub DeleteAllFibers()

Dim i As Long

For i = 1 To Data.MaxFiberCount
    If (Data.Fibers(i).Fiber <> 0) Then
        Call DeleteFiber(Data.Fibers(i).Id)
    End If
Next

End Sub

'Stellt sicher, dass alle Fiber aufgerumt sind. Muss von Fiber 0 aufgerufen werden
Public Sub CleanUp()

Dim i As Long

If (Data.CurrentFiber = 0) Then
    
    For i = 1 To (MaxFibers - 1)
        If (Data.Fibers(i).Fiber <> 0) Then
            Call DeleteFiber(Data.Fibers(i).Id)
        End If
    Next
    
End If

End Sub

'Lsst den aktuellen Fiber fr die angegebene Dauer in ms pausieren. Sleep(0) gibt die aktuelle Zeitscheibe auf
'und veranlasst ein unverzgliches wechseln zum nchsten Fiber
Public Sub Sleep(ByVal Duration As Long)

Dim i As Long

For i = 0 To (MaxFibers - 1)
    If (Data.Fibers(i).Fiber <> 0) Then
        If (Data.CurrentFiber = Data.Fibers(i).Id) Then
                    
            If (Data.FiberCount = 0) Then
                'Dies ist der einzige Fiber => nur warten
                Call W32Sleep(Duration)
            Else
                'Kontext wechseln
                
                Call GetSystemTimeAsFileTime(Data.Fibers(i).LastTimeSlice)
        
                'Nchster Ablaufzeitpunkt ist sofort
                Data.NextSchedule = 0
                
                Call GetSystemTimeAsFileTime(Data.Fibers(i).SleepTimeout)
                Data.Fibers(i).SleepTimeout = (Data.Fibers(i).SleepTimeout + Duration)
                
                Call Schedule
            End If
            
            Exit For
        
        End If
    End If
Next

End Sub

